#declare constants related to planet sign placement 
DEBILITATED = "Debilitated / Neecha"
ENEMYSIGN = "Enemy sign / Shatru rashi"
NEUTRALSIGN = "Neutral sign / Sama rashi"
FRIENDSIGN = "Friends sign / Mitra rashi"
OWNSIGN = "Own sign / Swa rashi"
MOOLTRIKONA = "Prime sign / Moolatrikona rashi"
EXHALTED = "Exhalted / Uchha"

#declare constants related to nature of house
DHARMA = "Dharma"
ARTHA = "Artha"
KAMA = "Kama"
MOKSHA = "Moksha"

#declare constants related to nature of planet
PAAPAGRAHA = "Krura graha"
PUNYAGRAHA = "Soumya graha"

#declare constants related to category of planet
DEVA = "Deva"
DANAVA = "Danava"
NEUTRAL = "Neutral"

#declare constants related to gender of planet
MALE = "Male"
FEMALE = "Female"
NEUTER = "Enuch"

#declare constants related to status of objects
INIT = 1
COMPUTED = 2
PARTIAL = 3
UNINIT = 4

#declare constants related to tattvas
FIRE = "Fire/Agni"
EARTH = "Earth/Pruthvi"
AIR = "Air/Vaayu"
WATER = "Water/Jala"

#declare constants related to varna of planets
BRAHMIN = "Brahmana"
KSHATRIYA = "Kshatriya"
VAISHYA = "Vaishya"
SHUDRA = "Shudra"

#declare constants related to Guna 
SATVA = "Sathvika"
RAJAS = "Rajasika"
TAMAS = "Tamasika"

#declare constants related to YogaDosha
PRESENT = "Present"
COMPRO = "Compromises" 
ABS = "Absent"

#declare constants related to planet relationship
MOOL = "Mooltrikona Rashi / Mooltrikon Sign"
SWAYAM = "Swayam Rashi / Own Sign"
ATHIMITRA = "Athimitra / Best Friend"
MITRA = "Mitra / Friend"
SAMA = "Sama / Neutral"
SHATRU = "Shatru / Enemy"
ATHISHATRU = "Athishatru / Worst Enemy"
 