from setuptools import setup

setup(
    name='parashara',
    packages=['parashara'],
    description='parashara',
    version='0.6',
    url='https://github.com/digvijay2021/jyotishyamitra',
    author='diggi',
    author_email='diggi.gkp@gmail.com',
    keywords=['pip','jyotish']
    )